
      SUBROUTINE BC
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'PAREQ.INC'
      CHARACTER*5 CHTYPE(2)
      DATA CHTYPE(1)/'Round'/
      DATA CHTYPE(2)/'Quadr'/
      CALL FIXPAR
C Read frequency
      OPEN(FILE='INPUT.BC',UNIT=33)
      OPEN(FILE='OUTPUT.BC',UNIT=34)
      READ(33,605,ERR=110)FREQ
 605  FORMAT(F10.3)
      WRITE(34,607)FREQ
 607  FORMAT(' Boom corrections for',F10.3,' MHz',/,' ',/,
     &' Boom    Boom     Wall    Hole   Element   Element  Dist to ',
     &'  Boom  Corrected',/,
     &' shape     D   thickness  diam     diam     length  boom end',
     &'  corr.  element',/,
     &'         (mm)     (mm)    (mm)     (mm)      (mm)     (mm)  ',
     &'  (mm)    (mm)')
C The BOM() subroutine takes dimensions in meters and assumes that
C the frequency of operation is 144MHz.
C Make DF the appropriate scaling factor.
      DF=FREQ/144000.
 200  READ(33,610,ERR=210)ITYP,D,TH,H,ED,X,Z
 610  FORMAT(I1,6F10.3)
      IF(ITYP.EQ.0.AND.D.EQ.0.AND.TH.EQ.0.AND.H.EQ.0)GOTO 220
      DS=DF*D
      EDS=DF*ED
      XS=DF*X
      HS=DF*H
      IF(DS.GT.0.09)THEN
        WRITE(*,*)'BOOM TUBE OUTER DIAMETER TOO LARGE',D
        STOP
      ENDIF
      IF(DS.LT.0.015)THEN
         WRITE(*,*)'BOOM TUBE DIAMETER TOO SMALL',D
         STOP
      ENDIF
      IF(EDS.GT.0.014)THEN
        WRITE(*,*)'ELEMENT TOO THICK',ED
        STOP
      ENDIF
      IF(EDS.LT.0.0028)THEN
        WRITE(*,*)'ELEMENT TOO THIN',ED
        STOP
      ENDIF
      IF(D/ED.LT.1.9)THEN
        WRITE(*,*)'ELEMENT TOO THICK FOR BOOM TUBE',ED,D
        STOP
      ENDIF
      IF(D/H.LT.1.7)THEN
        WRITE(*,*)'HOLE TOO BIG FOR BOOM TUBE',H,D
        STOP
      ENDIF
      IF(HS-EDS.LT.0.0003)THEN
        WRITE(*,*)'HOLE TOO SMALL FOR ELEMENT',H,ED
        STOP
      ENDIF
      DLEN=BOM(DS,EDS,DF*TH,HS,XS,ITYP)
      FQ=ZERFQ(XS,EDS)
      IF(FQ.LT.125)THEN
        WRITE(*,*)'ELEMENT TOO LONG',X
        STOP
      ENDIF
      IF(FQ.GT.160)THEN
        WRITE(*,*)'ELEMENT TOO SHORT',X
        STOP
      ENDIF
      IF(Z.LE.0)GOTO 300
C We have a value for the distance to the boom tube end.
C ENDCUT() wants dimensions in wavelengths at frequency FQ
      WLAM=CVEL/FQ
      DW=DS/WLAM
      ZS=Z*DF
      IF(Z.LT.D)THEN
        WRITE(*,*)'BOOM END TOO CLOSE (CORRECTION NOT YET KNOWN)',Z
        STOP
      ENDIF
      ZW=ZS/WLAM
      CALL ENDCUT(FC,DW,ZW,ITYP)
C Find the length adjustment that would cause the same frequency shift.
      DX=-0.001
      X1=XS
      X2=XS+DX
      F1=0
      F2=ZERFQ(X2,EDS)-FQ
 250  DER=(F2-F1)/DX
      X2=X1
      F2=F1
      X1=X1+(FC-F1)/DER
      F1=ZERFQ(X1,EDS)-FQ
      DX=X2-X1
      IF(ABS(DX).GT.0.0001)GOTO 250
      DLEN=DLEN+X1-XS
 300  CONTINUE
C Scale the correction in meters at 144MHz back to mm at the
C desired frequency
      DLEN=DLEN/DF
      IF(FREQ.LT.200)THEN
        WRITE(*,680)CHTYPE(ITYP+1),D,TH,H,ED,X,Z,DLEN,X+DLEN
        WRITE(34,680)CHTYPE(ITYP+1),D,TH,H,ED,X,Z,DLEN,X+DLEN
 680    FORMAT(' ',A5,F7.1,F8.1,F9.1,F9.2,F10.1,F9.1,F8.1,F10.1)
        ELSE
        WRITE(*,681)CHTYPE(ITYP+1),D,TH,H,ED,X,Z,DLEN,X+DLEN
        WRITE(34,681)CHTYPE(ITYP+1),D,TH,H,ED,X,Z,DLEN,X+DLEN
 681    FORMAT(' ',A5,F7.2,F8.2,F9.2,F9.3,F10.2,F9.2,F8.2,F10.2)
        ENDIF
      GOTO 200
 210  STOP
 220  WRITE(*,*)'  --  result written to OUTPUT.BC --'
      STOP
 110  WRITE(*,*)'ERROR first line of INPUT.BC incorrect'
      STOP
      END

      SUBROUTINE ENDCUT(ZE,DB,DIST,ITYP)
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'PAREQ.INC'
C ************* FREQUENCY SHIFT INSIDE CAGE ***********
C Endcut estimates the frequency shift caused by cutting the boom tube
C at a distance DIST from the element on one side.
C The reference is the frequency with a long tube that fills the cage.
C DB is the boom tube diameter, ITYP=0 for round and ITYP=1 for square
C cross section.
C******************************************************
      D=DB*(1+ITYP*SQUARE_ADD)
      ZE=FAR_FAC*D**2*DIST**FAR_DIST_EXP
      RETURN
      END


      FUNCTION ZERFQ(X,D)
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'PAREQ.INC'
C ********** FREQUENCY IN CAGE **************
C This routine estimates the frequency inside the cage for an
C element without any boom tube.
C The input is length and diameter in meters
C *******************************************
      T=(1-ZER_LEN)*X+ZER_DIAM*SQRT(D)+ZER_CONST
      ZERFQ=0.5*CVEL/T
      RETURN
      END

      SUBROUTINE FIXPAR
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'PAREQ.INC'
      ZER_CONST=0.13405D0
      ZER_LEN=0.0547D0
      ZER_DIAM=0.733D0
      FAR_FAC=23.D0
      FAR_DIST_EXP=-0.887D0
      SQUARE_ADD=0.18D0
      H32=1.5193D0
      H31=-41.75D0
      H33=-36.9D0
      H22=1.135D0
      H20=0.02285D0
      H21=4.78D0
      H23=0.0204D0
      H12=-0.008620
      H41=0.16D0
      RETURN
      END

      FUNCTION BOM(DB,ELD,T,H,XL,ITYP)
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'PAREQ.INC'
C *********  THROUGH HOLE MOUNTED ELEMENT FREQUENCY SHIFT *********
C Dimensions in meters
C DB = Boom diameter
C ELD = Element diameter
C T = Boom tube wall thickness
C H = Hole diameter
C XL = Element length
C **************************************************************
      D=DB*(1+ITYP*SQUARE_ADD)
      BOM=(SQRT(D)+H12/SQRT(D))
     & *(H20+H21*T+H22*(H-ELD)+H23*(ELD/D))
     & *(1+H31*(H-ELD)+H32*SQRT(DB-2*T)+H33*(ELD*T/D))
     & *(H41+XL)
      RETURN
      END
